==> PyRQA-4.0.0.tar.gz: PyRQA-4.0.0/pyrqa/operators/detect_vertical_lines/radius/column_no_materialisation.py <==
#!/usr/bin/env python

"""
Detect vertical lines.

Input data format: Column store
Recurrence matrix materialisation: No
"""

import numpy as np
import pyopencl as cl

from pyrqa.neighbourhood import FixedRadius, \
    RadiusCorridor
from pyrqa.opencl import OpenCL
from pyrqa.runtimes import OperatorRuntimes

__author__ = "Tobias Rawald"
__copyright__ = "Copyright 2015, 2018, 2019 The PyRQA project"
__credits__ = ["Tobias Rawald",
               "Mike Sips"]
__license__ = "Apache-2.0"
__maintainer__ = "Tobias Rawald"
__email__ = "pyrqa@gmx.net"
__status__ = "Development"


def detect_vertical_lines(settings,
                          sub_matrix,
                          device,
                          context,
                          command_queue,
                          kernels,
                          return_time_series_buffers=False):
    """
    :param settings: Settings.
    :param sub_matrix: Sub matrix.
    :param device: OpenCL device.
    :param context: OpenCL context.
    :param command_queue: OpenCL command queue.
    :param kernels: OpenCL kernels.
    :param return_time_series_buffers: Shall the OpenCL time series buffers be returned? (Default: False)
    :return: return_time_series_buffers==False: Runtimes / return_time_series_buffers==True: OpenCL time series buffer X dimension, OpenCL time series buffer X dimension, runtimes.
    """
    transfer_to_device_events = []
    execute_computations_events = []
    transfer_from_device_events = []

    detect_vertical_lines_kernel = kernels[0]

    # Write to buffers
    time_series_x = settings.time_series_x.get_sub_series(sub_matrix.start_x,
                                                          sub_matrix.dim_x)

    time_series_x_buffer = cl.Buffer(context,
                                     cl.mem_flags.READ_ONLY,
                                     time_series_x.size * time_series_x.itemsize)

    transfer_to_device_events.append(cl.enqueue_copy(command_queue,
                                                     time_series_x_buffer,
                                                     time_series_x,
                                                     device_offset=0,
                                                     wait_for=None,
                                                     is_blocking=False))

    time_series_y = settings.time_series_y.get_sub_series(sub_matrix.start_y,
                                                          sub_matrix.dim_y)

    time_series_y_buffer = cl.Buffer(context,
                                     cl.mem_flags.READ_ONLY,
                                     time_series_y.size * time_series_y.itemsize)

    transfer_to_device_events.append(cl.enqueue_copy(command_queue,
                                                     time_series_y_buffer,
                                                     time_series_y,
                                                     device_offset=0,
                                                     wait_for=None,
                                                     is_blocking=False))

    recurrence_points_buffer = cl.Buffer(context,
                                         cl.mem_flags.READ_WRITE,
                                         sub_matrix.recurrence_points.size * sub_matrix.recurrence_points.itemsize)

    transfer_to_device_events.append(cl.enqueue_copy(command_queue,
                                                     recurrence_points_buffer,
                                                     sub_matrix.recurrence_points,
                                                     device_offset=0,
                                                     wait_for=None,
                                                     is_blocking=False))

    vertical_carryover_buffer = cl.Buffer(context,
                                          cl.mem_flags.READ_WRITE,
                                          sub_matrix.vertical_length_carryover.size * sub_matrix.vertical_length_carryover.itemsize)

    transfer_to_device_events.append(cl.enqueue_copy(command_queue,
                                                     vertical_carryover_buffer,
                                                     sub_matrix.vertical_length_carryover,
                                                     device_offset=0,
                                                     wait_for=None,
                                                     is_blocking=False))

    white_vertical_carryover_buffer = cl.Buffer(context,
                                                cl.mem_flags.READ_WRITE,
                                                sub_matrix.white_vertical_length_carryover.size * sub_matrix.white_vertical_length_carryover.itemsize)

    transfer_to_device_events.append(cl.enqueue_copy(command_queue,
                                                     white_vertical_carryover_buffer,
                                                     sub_matrix.white_vertical_length_carryover,
                                                     device_offset=0,
                                                     wait_for=None,
                                                     is_blocking=False))

    vertical_frequency_distribution_buffer = cl.Buffer(context,
                                                       cl.mem_flags.READ_WRITE,
                                                       sub_matrix.vertical_frequency_distribution.size * sub_matrix.vertical_frequency_distribution.itemsize)

    transfer_to_device_events.append(cl.enqueue_copy(command_queue,
                                                     vertical_frequency_distribution_buffer,
                                                     sub_matrix.vertical_frequency_distribution,
                                                     device_offset=0,
                                                     wait_for=None,
                                                     is_blocking=False))

    white_vertical_frequency_distribution_buffer = cl.Buffer(context,
                                                             cl.mem_flags.READ_WRITE,
                                                             sub_matrix.white_vertical_frequency_distribution.size * sub_matrix.white_vertical_frequency_distribution.itemsize)

    transfer_to_device_events.append(cl.enqueue_copy(command_queue,
                                                     white_vertical_frequency_distribution_buffer,
                                                     sub_matrix.white_vertical_frequency_distribution,
                                                     device_offset=0,
                                                     wait_for=None,
                                                     is_blocking=False))

    # Execute detect vertical lines kernel
    if type(settings.neighbourhood) is FixedRadius:
        detect_vertical_lines_args = [time_series_x_buffer,
                                      time_series_y_buffer,
                                      np.uint32(sub_matrix.dim_x),
                                      np.uint32(sub_matrix.dim_y),
                                      np.uint32(settings.embedding_dimension),
                                      np.uint32(settings.time_delay),
                                      settings.dtype(settings.neighbourhood.condition),
                                      recurrence_points_buffer,
                                      vertical_frequency_distribution_buffer,
                                      vertical_carryover_buffer,
                                      white_vertical_frequency_distribution_buffer,
                                      white_vertical_carryover_buffer]
    elif type(settings.neighbourhood) is RadiusCorridor:
        detect_vertical_lines_args = [time_series_x_buffer,
                                      time_series_y_buffer,
                                      np.uint32(sub_matrix.dim_x),
                                      np.uint32(sub_matrix.dim_y),
                                      np.uint32(settings.embedding_dimension),
                                      np.uint32(settings.time_delay),
                                      settings.dtype(settings.neighbourhood.condition[0]),
                                      settings.dtype(settings.neighbourhood.condition[1]),
                                      recurrence_points_buffer,
                                      vertical_frequency_distribution_buffer,
                                      vertical_carryover_buffer,
                                      white_vertical_frequency_distribution_buffer,
                                      white_vertical_carryover_buffer]

    OpenCL.set_kernel_args(detect_vertical_lines_kernel, detect_vertical_lines_args)

    global_work_size = [int(sub_matrix.dim_x + (device.max_work_group_size - (sub_matrix.dim_x % device.max_work_group_size)))]
    local_work_size = None

    execute_computations_events.append(cl.enqueue_nd_range_kernel(command_queue,
                                                                  detect_vertical_lines_kernel,
                                                                  global_work_size,
                                                                  local_work_size))

    command_queue.finish()

    # Read from buffers
    transfer_from_device_events.append(cl.enqueue_copy(command_queue,
                                                       sub_matrix.recurrence_points,
                                                       recurrence_points_buffer,
                                                       device_offset=0,
                                                       wait_for=None,
                                                       is_blocking=False))

    transfer_from_device_events.append(cl.enqueue_copy(command_queue,
                                                       sub_matrix.vertical_length_carryover,
                                                       vertical_carryover_buffer,
                                                       device_offset=0,
                                                       wait_for=None,
                                                       is_blocking=False))

    transfer_from_device_events.append(cl.enqueue_copy(command_queue,
                                                       sub_matrix.white_vertical_length_carryover,
                                                       white_vertical_carryover_buffer,
                                                       device_offset=0,
                                                       wait_for=None,
                                                       is_blocking=False))

    transfer_from_device_events.append(cl.enqueue_copy(command_queue,
                                                       sub_matrix.vertical_frequency_distribution,
                                                       vertical_frequency_distribution_buffer,
                                                       device_offset=0,
                                                       wait_for=None,
                                                       is_blocking=False))

    transfer_from_device_events.append(cl.enqueue_copy(command_queue,
                                                       sub_matrix.white_vertical_frequency_distribution,
                                                       white_vertical_frequency_distribution_buffer,
                                                       device_offset=0,
                                                       wait_for=None,
                                                       is_blocking=False))

    command_queue.finish()

    # Determine runtimes
    runtimes = OperatorRuntimes()
    runtimes.transfer_to_device += OpenCL.convert_events_runtime(transfer_to_device_events)
    runtimes.execute_computations += OpenCL.convert_events_runtime(execute_computations_events)
    runtimes.transfer_from_device += OpenCL.convert_events_runtime(transfer_from_device_events)

    if return_time_series_buffers:
        return time_series_x_buffer, \
               time_series_y_buffer, \
               runtimes

    return runtimes

==> PyRQA-7.0.1.tar.gz: PyRQA-7.0.1/pyrqa/operators/detect_vertical_lines/radius/column_no_materialisation.py <==
#!/usr/bin/env python

"""
Detect vertical lines.

Input data format: Column store
Recurrence matrix materialisation: No
"""

import numpy as np
import pyopencl as cl

from pyrqa.neighbourhood import FixedRadius, \
    RadiusCorridor
from pyrqa.opencl import OpenCL
from pyrqa.runtimes import OperatorRuntimes

__author__ = "Tobias Rawald"
__copyright__ = "Copyright 2015-2020 The PyRQA project"
__credits__ = ["Tobias Rawald",
               "Mike Sips"]
__license__ = "Apache-2.0"
__maintainer__ = "Tobias Rawald"
__email__ = "pyrqa@gmx.net"
__status__ = "Development"


def detect_vertical_lines(settings,
                          sub_matrix,
                          device,
                          context,
                          command_queue,
                          kernels,
                          return_time_series_buffers=False):
    """
    :param settings: Settings.
    :param sub_matrix: Sub matrix.
    :param device: OpenCL device.
    :param context: OpenCL context.
    :param command_queue: OpenCL command queue.
    :param kernels: OpenCL kernels.
    :param return_time_series_buffers: Shall the OpenCL time series buffers be returned? (Default: False)
    :return: return_time_series_buffers==False: Runtimes / return_time_series_buffers==True: OpenCL time series buffer X dimension, OpenCL time series buffer X dimension, runtimes.
    """
    transfer_to_device_events = []
    execute_computations_events = []
    transfer_from_device_events = []

    detect_vertical_lines_kernel = kernels[0]

    # Write to buffers
    time_series_x = settings.time_series_x.get_sub_series(sub_matrix.start_x,
                                                          sub_matrix.dim_x)

    time_series_x_buffer = cl.Buffer(context,
                                     cl.mem_flags.READ_ONLY,
                                     time_series_x.size * time_series_x.itemsize)

    transfer_to_device_events.append(cl.enqueue_copy(command_queue,
                                                     time_series_x_buffer,
                                                     time_series_x,
                                                     device_offset=0,
                                                     wait_for=None,
                                                     is_blocking=False))

    time_series_y = settings.time_series_y.get_sub_series(sub_matrix.start_y,
                                                          sub_matrix.dim_y)

    time_series_y_buffer = cl.Buffer(context,
                                     cl.mem_flags.READ_ONLY,
                                     time_series_y.size * time_series_y.itemsize)

    transfer_to_device_events.append(cl.enqueue_copy(command_queue,
                                                     time_series_y_buffer,
                                                     time_series_y,
                                                     device_offset=0,
                                                     wait_for=None,
                                                     is_blocking=False))

    recurrence_points_buffer = cl.Buffer(context,
                                         cl.mem_flags.READ_WRITE,
                                         sub_matrix.recurrence_points.size * sub_matrix.recurrence_points.itemsize)

    transfer_to_device_events.append(cl.enqueue_copy(command_queue,
                                                     recurrence_points_buffer,
                                                     sub_matrix.recurrence_points,
                                                     device_offset=0,
                                                     wait_for=None,
                                                     is_blocking=False))

    vertical_carryover_buffer = cl.Buffer(context,
                                          cl.mem_flags.READ_WRITE,
                                          sub_matrix.vertical_length_carryover.size * sub_matrix.vertical_length_carryover.itemsize)

    transfer_to_device_events.append(cl.enqueue_copy(command_queue,
                                                     vertical_carryover_buffer,
                                                     sub_matrix.vertical_length_carryover,
                                                     device_offset=0,
                                                     wait_for=None,
                                                     is_blocking=False))

    white_vertical_carryover_buffer = cl.Buffer(context,
                                                cl.mem_flags.READ_WRITE,
                                                sub_matrix.white_vertical_length_carryover.size * sub_matrix.white_vertical_length_carryover.itemsize)

    transfer_to_device_events.append(cl.enqueue_copy(command_queue,
                                                     white_vertical_carryover_buffer,
                                                     sub_matrix.white_vertical_length_carryover,
                                                     device_offset=0,
                                                     wait_for=None,
                                                     is_blocking=False))

    vertical_frequency_distribution_buffer = cl.Buffer(context,
                                                       cl.mem_flags.READ_WRITE,
                                                       sub_matrix.vertical_frequency_distribution.size * sub_matrix.vertical_frequency_distribution.itemsize)

    transfer_to_device_events.append(cl.enqueue_copy(command_queue,
                                                     vertical_frequency_distribution_buffer,
                                                     sub_matrix.vertical_frequency_distribution,
                                                     device_offset=0,
                                                     wait_for=None,
                                                     is_blocking=False))

    white_vertical_frequency_distribution_buffer = cl.Buffer(context,
                                                             cl.mem_flags.READ_WRITE,
                                                             sub_matrix.white_vertical_frequency_distribution.size * sub_matrix.white_vertical_frequency_distribution.itemsize)

    transfer_to_device_events.append(cl.enqueue_copy(command_queue,
                                                     white_vertical_frequency_distribution_buffer,
                                                     sub_matrix.white_vertical_frequency_distribution,
                                                     device_offset=0,
                                                     wait_for=None,
                                                     is_blocking=False))

    # Execute detect vertical lines kernel
    if type(settings.neighbourhood) is FixedRadius:
        detect_vertical_lines_args = [time_series_x_buffer,
                                      time_series_y_buffer,
                                      np.uint32(sub_matrix.dim_x),
                                      np.uint32(sub_matrix.dim_y),
                                      np.uint32(settings.embedding_dimension),
                                      np.uint32(settings.time_series_x.time_delay),
                                      np.uint32(settings.time_series_y.time_delay),
                                      settings.dtype(settings.neighbourhood.condition),
                                      recurrence_points_buffer,
                                      vertical_frequency_distribution_buffer,
                                      vertical_carryover_buffer,
                                      white_vertical_frequency_distribution_buffer,
                                      white_vertical_carryover_buffer]
    elif type(settings.neighbourhood) is RadiusCorridor:
        detect_vertical_lines_args = [time_series_x_buffer,
                                      time_series_y_buffer,
                                      np.uint32(sub_matrix.dim_x),
                                      np.uint32(sub_matrix.dim_y),
                                      np.uint32(settings.embedding_dimension),
                                      np.uint32(settings.time_series_x.time_delay),
                                      np.uint32(settings.time_series_y.time_delay),
                                      settings.dtype(settings.neighbourhood.condition[0]),
                                      settings.dtype(settings.neighbourhood.condition[1]),
                                      recurrence_points_buffer,
                                      vertical_frequency_distribution_buffer,
                                      vertical_carryover_buffer,
                                      white_vertical_frequency_distribution_buffer,
                                      white_vertical_carryover_buffer]

    OpenCL.set_kernel_args(detect_vertical_lines_kernel, detect_vertical_lines_args)

    global_work_size = [int(sub_matrix.dim_x + (device.max_work_group_size - (sub_matrix.dim_x % device.max_work_group_size)))]
    local_work_size = None

    execute_computations_events.append(cl.enqueue_nd_range_kernel(command_queue,
                                                                  detect_vertical_lines_kernel,
                                                                  global_work_size,
                                                                  local_work_size))

    command_queue.finish()

    # Read from buffers
    transfer_from_device_events.append(cl.enqueue_copy(command_queue,
                                                       sub_matrix.recurrence_points,
                                                       recurrence_points_buffer,
                                                       device_offset=0,
                                                       wait_for=None,
                                                       is_blocking=False))

    transfer_from_device_events.append(cl.enqueue_copy(command_queue,
                                                       sub_matrix.vertical_length_carryover,
                                                       vertical_carryover_buffer,
                                                       device_offset=0,
                                                       wait_for=None,
                                                       is_blocking=False))

    transfer_from_device_events.append(cl.enqueue_copy(command_queue,
                                                       sub_matrix.white_vertical_length_carryover,
                                                       white_vertical_carryover_buffer,
                                                       device_offset=0,
                                                       wait_for=None,
                                                       is_blocking=False))

    transfer_from_device_events.append(cl.enqueue_copy(command_queue,
                                                       sub_matrix.vertical_frequency_distribution,
                                                       vertical_frequency_distribution_buffer,
                                                       device_offset=0,
                                                       wait_for=None,
                                                       is_blocking=False))

    transfer_from_device_events.append(cl.enqueue_copy(command_queue,
                                                       sub_matrix.white_vertical_frequency_distribution,
                                                       white_vertical_frequency_distribution_buffer,
                                                       device_offset=0,
                                                       wait_for=None,
                                                       is_blocking=False))

    command_queue.finish()

    # Determine runtimes
    runtimes = OperatorRuntimes()
    runtimes.transfer_to_device += OpenCL.convert_events_runtime(transfer_to_device_events)
    runtimes.execute_computations += OpenCL.convert_events_runtime(execute_computations_events)
    runtimes.transfer_from_device += OpenCL.convert_events_runtime(transfer_from_device_events)

    if return_time_series_buffers:
        return time_series_x_buffer, \
               time_series_y_buffer, \
               runtimes

    return runtimes
